; created 12/06/91
; updated 09/30/92

;---------------------------------------------
; Based in part on public domain routines by Charles Lazo III
; released in 1989. Updated by Sidney J. Kelly
;---------------------------------------------
;=======================================================================
;   Balance 
;	Copyright (C) Copr. 1992, 1993 by Sidney J. Kelly
;           All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;=======================================================================

DOSSEG
.MODEL MEDIUM, BASIC
.CODE

;======================================================================
;DECLARE FUNCTION DATE2JUL& (BYVAL Month%, BYVAL Day%, BYVAL Year%)
; Inputs:
;	Month  1-12
;	Day    1-31
;	Year   1600 to 4000
; Returns:
;	LONG INT Julian date number, if valid date
;	else, if invalid date, then 0
;
;  About 2.4 times faster than QBASIC version of program and better
;       error checking!
;======================================================================
; RM120TS8,16,24,32,40

EVEN
Month	dw	0	; validated month (0-11)
Day	dw	0	; validated day
Year	dw	0	; validated year
BeginDate dd    2299161	; Julian Day offset for the Gregorian Calendar.
			; It's the Julian Day for first day of Gregorian 
			; calendar, 15-Oct-1582
LeapDays  dw	0	; additional days to add because of leap years

; check incoming data   (range of 0 to 11)
Leap_Table 	db	31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
Non_Leap_Table	db 	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31

EVEN                    ; speed 8086, 80286,80386 access
; count elapsed time
; Number of days up to a given month number (range 0 - 11) for a non-leap year.
days_nly	dw	0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334

EVEN
; Number of days up to a given month number (range 0 - 11) for a leap year.
days_ly		dw	0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335

Leapyr	db	0	; 0 if not a leap year, 1 if is a leap year

EVEN                    ; speed 8086, 80286,80386 access
DateErr:
	Jmp     ErrExit         ; put here for a near jump
EVEN                    ; speed 8086, 80286,80386 access
DATE2JUL PROC FAR BASIC MONTH:WORD, DAY:WORD, YEAR:WORD
;check year
	Mov     AX,YEAR         ; get year from stack (note BYVAL)
	Cmp	AX,100		; did person assume 20th century?
	Ja	@f		; no, so continue testing
	Add	AX,1900		; else, make it 20th century
@@:
	Cmp	AX,3999		; is year > 3999?
	Ja      DateErr         ; out of range so exit
	Cmp	AX,1600		; is it > 1599
	Jb      DateErr         ; no so exit
	Mov	Year,AX		; store input value 
	Call	Near Ptr IS_LEAP? ; check leap year stat for input check
	Mov	DX,AX		; store year in DX too
;check month
	Mov     BX,MONTH        ; get Month from stack (note BYVAL)
	Cmp	BX,12		; is > 12?
	Ja      DateErr         ; yes, error
	Or	BX,BX		; is it 0
	Jz      DateErr         ; yes, error
	Dec     BX              ; make MONTH zero biased
	Mov     Month,BX        ; save value in memory
	Xor     CH,CH           ; clear high byte; use low byte only
	Mov     CL,Leap_Table[BX]  ; assume leap yr, get size of this month
	Cmp     Leapyr,0        ; is it a leap year?
	Jne     Check_Day       ; yes, so skip ahead
	Mov     CL,Non_Leap_Table[BX] ; get size of this month
Check_Day:
	Mov     AX,DAY          ; get day value (note BYVAL)
	Or	AX,AX		; is it zero?
	Jz      DateErr         ; out of range
	Cmp     AX,CX           ; is day out of range
	Ja      DateErr         ; too big, so quit
	Mov	Day,AX		; save value
	Mov     AX,DX           ; get year from DX
	Sub	AX,1580		; make it biased by 1580
	Shr	AX,1		;   every year after 1582 that is a
	Shr	AX,1		;   multiple four is a leap year
	Mov     LeapDays,AX     ;   and save it as partial result
	Mov	AX,Year		; get year again
	Sub	AX,1500		;   calculate number of centuries
	Mov	BX,100		;   since the year 1500
	Xor	DX,DX		;   clear high word
	Div	BX
	Sub     LeapDays,AX     ; ends of centuries are not leap years
	Mov	AX,Year		;   but, every century year divisible
	Sub	AX,1200		;   by 400 is a leap year, so we
	Mov	BX,400		;   determine their number and add them
	Xor	DX,DX		;   back to the accumulation
	Div	BX
	Add     LeapDays,AX
	Mov	AX,Year		; now to calculate the number of days
	Sub	AX,1582		;   that have elapsed between the date
	Dec	AX
	Mov	BX,365		;   passed and 15-Oct-1582
	Mul	BX		; DX:AX has number of days due to years
	Add     AX,LeapDays     ; add in leap days
	Adc	DX,0
	Add	AX,16+30+31	; add number of days to complete 1582
	Adc	DX,0		; increment DX if overflow above
; Add months
	Mov	BX,Month	; add number of elapsed days in prior month
	Add	BX,BX		; multiply BX by 2 to get words
	Add	AX,days_nly[BX]	; get index value
	Adc	DX,0		; increment DX if overflow above
	Add	AX,Day		; add days of current month
	Adc	DX,0		; increment DX if overflow above
	Cmp	Leapyr,0	; is it a leap year?
	JE	AddBase		; no, so jmp ahead
	Cmp	Month,1		; is it before March?
	Ja      AddBase         ; no, jump ahead
	Sub	AX,1		; remove leap day
	Sbb	DX,0
AddBase:
	Mov	BX,Word Ptr BeginDate ; get low word in BX
	Mov	CX,Word Ptr BeginDate+2 ; high word in CX
	Add	AX,BX		; add days since base year - low word
	Adc	DX,CX		; add high word
	Ret
ErrExit:
	Xor     AX,AX           ; return 0 for DX:AX
	Mov     DX,AX
	Ret
DATE2JUL ENDP

;======================================================================
;DECLARE SUB JUL2DATE (Month%, Day%, Year%, JulNum&)
; Input:
;       a Julian date number, JulNum&, after 12/31/1599 and before 1/1/4000
; Output:
;       Month, Day, and Year represented by input number
;
; About 5.2 times faster than a QBASIC version & better error checking!
;======================================================================

EVEN
Years	dw	0	; hold interim values
Days	dw	0,0

EVEN                            ; speed 8086, 80286,80386 access
JulErr:                         ; placed here for near jumps
	Jmp     Err_Quit
EVEN                            ; speed 8086, 80286,80386 access
JUL2DATE PROC FAR USES SI, MONTH:WORD, DAY:WORD, YEAR:WORD, JULNUM:WORD
	Mov     BX,JULNUM       ; get input value
	Mov     AX,[BX]         ; into DX:AX
	Mov	DX,[BX+2]
	Cmp     DX,23h          ; is date < 1/1/1600
	Ja      @f
	Cmp     AX,02DA8h
	Jb      JulErr          ; yes exit w/ error
				; prevents divide by zero error
@@:
	Cmp     DX,30h          ; is date > 12/31/3999
	Jb      @f              ; w/in range ok
	Ja      JulErr          ; out of range, exit
	Cmp     AX,08DCEh
	Jae     JulErr          ; out of range so exit
@@:
	Sub     AX, word ptr BeginDate
	Sbb	DX, word ptr BeginDate+2
	JS      JulErr          ; Input num < base number?, sign bit set
	Sub	AX,16+30+31	; subtract number of days for the
	Sbb	DX,0		;   remainder of 1582 making 1-Jan-1583
	Mov	Days,AX		;   the base date
	Mov	Days+2,DX	;   days from base date stored
	Mov	BX,366		; we divide by 366 to obtain a close
	Div	BX		;   lower bound on the number of
	Mov	Years,AX	;   years from the base date
	Mov	BX,365		; now, deduct these years from the
	Mul	BX		;   tally of days
; With the maximum Julian Day acceptable as 9,999,999 the number of days that
; remain after this subtraction will fit in a word size variable.  Thus, from
; this point on only the first word of the days variable is used.
	Sub     Days,AX
; Next, deduct the number of intercalated days (leap days) accompaning the 365
; day years already deducted.
	Mov	AX,Years	; get number of years deducted
	Add	AX,2		; by 1-Jan-1585 (years = 2) one leap
	Shr	AX,1		;   year has occured so 2 => 4
	Shr	AX,1		; divide by 4 to obtain number of days
	Sub	Days,AX		;   due to leap years and subtract
	Mov	AX,Years	; get number of years deducted
	Add	AX,82		; this translates a years value of 18
	Mov	BX,100		;   to 100 signifying 1-Jan-1600
	Xor	DX,DX		; (clear high word)
	Div	BX		; years divisible by 100 are not leaps
	Add	Days,AX		;   so add back those just deducted
	Mov	AX,Years	; get number of years deducted
	Add	AX,382		; this translates a years value of 18
	Mov	BX,400		;   to 400 (signifying 31-Dec-1600),
	Xor	DX,DX		;   the first year after 1582 divisible
	Div	BX		;   by 400.  They're leap years so
	Sub	Days,AX		;   subtract them again
; At this point we merely deduct whole years from the tally until it drops to
; 366 or below for a leap year or to 365 or below for a non leap year.
	Mov	AX,Years	; get number of years deducted
	Add	AX,1583		; Add base year
	Mov	Year,AX		; make that our provisional year
	Mov	AX,Days		; get days remaining till now
Get_next:
	Call	Near Ptr IS_LEAP? ; action depends upon leap year status
	JC      Get_leap        ; if carry then a leap year
	Cmp	AX,365		; a non leap year; days <= 365?
	Jbe     Get_mnly        ; yes, figure month for a non leap year
	Inc	Year		; adjust year
	Sub	AX,365		; and remaining days
	Jmp     Short   Get_next
Get_leap:
	Cmp	AX,366		; a leap year; days <= 366?
	Jbe     Get_mly         ; yes, figure month for a leap year
	Inc	Year		; adjust year
	Sub	AX,366		; and remaining days
	Jmp     Short   Get_next
Get_mnly:
	Xor	BX,BX		; start with month 0 (January)
	Xor	CH,CH		; clear high byte; use low byte only
Get_mnly1:
	Mov	CL,Non_Leap_Table[BX] ; get size of this month
	Cmp	AX,CX		; is this the right month?
	Jbe	j2d_store	; yes, store month and day
	Sub	AX,CX		; reduce count by this month's days
	Inc	BX		; try next month
	Jmp     SHORT   Get_mnly1
Get_mly:
	Xor	BX,BX		; start with month 0 (January)
	Xor	CH,CH		; clear high byte; use low byte only
Get_mly1:
	Mov	CL,Leap_Table[BX] ; get size of this month
	Cmp	AX,CX		; is this the right month?
	Jbe	j2d_store	; yes, store month and day
	Sub	AX,CX		; reduce count by this month's days
	Inc	BX		; try next month
	Jmp     SHORT   Get_mly1
j2d_store:
	Inc     BX              ; remove zero bias
	Mov     Month,BX        ; store month
	Mov	CX,BX		; store month in CX too
	Mov	Day,AX		; store day
	Mov	BX,DAY		; get address of DAY%
	Mov	[BX],AX		; store Day in DAY%
	Mov	BX,MONTH	; get address of MONTH%
	Mov	[BX],CX		; Store Month in MONTH%
	Mov	AX,Year		; get calculated year value
	Mov	BX,YEAR		; get address of YEAR%
	Mov	[BX],AX		; store value
	Ret
Err_Quit:                       ; just return error values
	Xor	AX,AX
	Mov	BX,MONTH
	Mov	[BX],AX
	Mov	BX,DAY
	Mov	[BX],AX
	Mov	BX,YEAR
	Mov	[BX],AX
	Ret
JUL2DATE ENDP

;======================================================================
;DECLARE FUNCTION SIZEOFMONTH% (BYVAL Month%, BYVAL Year%,)
; Input:
;       Year after 12/31/1599 and before 1/1/4000
;       Month between 1 and 12
; Output:
;       Number of days in month.
;       Error is reported by 0
; Assumes:
;       Year less than 100 is in 20th Century
;======================================================================

EVEN
SIZEOFMONTH PROC FAR BASIC USES SI, MONTH:WORD, YEAR:WORD
	Mov     AX,YEAR         ; get year from stack (note BYVAL)
	Cmp     AX,100          ; did person assume 20th century?
	Ja      @f              ; no, so continue testing
	Add     AX,1900         ; else, make it 20th century
@@:
	Cmp     AX,3999         ; is year > 3999?
	Ja      DateErr1        ; out of range so exit
	Cmp     AX,1600         ; is it > 1599
	Jb      DateErr1        ; no so exit
	Mov     Year,AX         ; store input value
	Mov     SI,MONTH        ; get month from stack (note BYVAL)
	Dec     SI              ; make it zero based
	Cmp     SI,11           ; see if w/in range 0-11
	Ja      DateErr1        ; out of range quit
	Call    Near Ptr IS_LEAP? ; check leap year status
	Assume	DS:NOTHING	; lie so OFFSET works
	Mov	BX,OFFSET Leap_Table	; guess leap year
	Assume  DS:@data        ; tell the truth
	Jc	SM_Get_Value	; assumed correctly, it is leap year
	Assume	DS:NOTHING	; lie so OFFSET works
	Mov	BX,OFFSET Non_Leap_Table	; it is non leap year
	Assume  DS:@data        ; tell the truth
SM_Get_Value:
	Xor     AH,AH           ; clear high byte; use low byte only
	Mov     AL,Byte Ptr CS:[BX+SI]	; get size of this month
Check_Day1:
	Ret
DateErr1:
	Xor     AX,AX           ; return we got an error
	Jmp     Short   Check_Day1
SIZEOFMONTH  ENDP

;--------------------------------------------
; Input:
;       Year contains the year value
; Output:
;       1 in Leapyr if a leap year, CARRY set
;       0 in Leapyr if it is not a leap year, CARRY clear
; Preserves:
;	AX,DX,CX
; Trashes:
;	BX
;--------------------------------------------

EVEN
IS_LEAP? Label NEAR		; use a label to keep name private
	Push	AX
	Push	DX
	Mov	AX,Year
	Sub	AX,1200		; determine if year number is evenly
	Mov	BX,400		; divisible by 400
	Xor	DX,DX		; clear high word for testing below
	Div	BX
	Or	DX,DX		; is remainder zero?
	Jz	yes_leap	; yes, it's a leap year
	Mov	AX,Year		; next check divisibility by 100
	Mov	BX,100
	Xor	DX,DX		; clear high word for testing below
	Div	BX
	Or	DX,DX		; is remainder zero?
	Jz	not_leap	; yes, divisible by 100 but not 400
	Mov	AX,Year		; finally, is it divisible by four?
	Mov	BX,4
	Xor	DX,DX		; clear high word for testing below
	Div	BX
	Or	DX,DX		; is remainder 0?
	Jz	yes_leap	; yes, it's a leap year
not_leap:
	Xor	AL,AL		; return not a leap year
	Mov	Leapyr,AL
	Pop	DX		; recover days accumulation
	Pop	AX
	Clc                     ; clear carry
	Retn
yes_leap:
	Mov	AL,1		; return it is a leap year
	Mov	Leapyr,AL
	Pop	DX		; recover days accumulation
	Pop	AX
	Stc                     ; set carry
	Retn
END

